"use strict";
var WebFont = require('webfontloader');
var Rx_1 = require('rxjs/Rx');
require("rxjs/add/operator/debounceTime");
require("rxjs/add/operator/distinctUntilChanged");
var forms_1 = require('@angular/forms');
var core_1 = require('@angular/core');
var font_picker_service_1 = require('./font-picker.service');
var interfaces_1 = require('./interfaces');
var DialogComponent = (function () {
    function DialogComponent(el, service) {
        this.el = el;
        this.service = service;
        this.styles = [];
        this.fontAmount = 5;
        this.loadedFonts = 0;
        this.presetFonts = [];
        this.googleFonts = [];
        this.currentFonts = [];
        this.dialogArrowSize = 10;
        this.dialogArrowOffset = 15;
        this.searchTerm = new forms_1.FormControl('');
        this.config = {
            suppressScrollX: true
        };
        this.loading = true;
        this.selectedFont = false;
        this.presetVisible = true;
    }
    DialogComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.searchTerm
            .valueChanges
            .debounceTime(500)
            .distinctUntilChanged()
            .subscribe(function (text) {
            if (!text) {
                _this.presetVisible = true;
                _this.listLabel = "Popular fonts";
            }
            else {
                _this.presetVisible = false;
                _this.listLabel = "Search results";
            }
            _this.searchGoogleFonts(text);
        });
        // Don't allow too many loading requests in a short time span.
        Rx_1.Observable.fromEvent(this.el.nativeElement, 'ps-y-reach-end')
            .debounceTime(150)
            .subscribe(function () { return _this.loadMoreFonts(); });
        this.testContainer = document.createElement('span');
        this.testContainer.innerHTML = Array(100).join('wi');
        this.testContainer.style.cssText = [
            'position: absolute',
            'width: auto',
            'font-size: 128px',
            'left: -99999px'
        ].join(' !important;');
        this.listenerResize = function () { return _this.onResize(); };
        this.listenerMouseDown = function (event) { return _this.onMouseDown(event); };
        this.openFontPicker();
    };
    DialogComponent.prototype.setDialog = function (instance, elementRef, font, fpPosition, fpPositionOffset, fpPositionRelativeToArrow, fpPresetLabel, fpPresetFonts, fpUploadButton, fpUploadButtonClass, fpUploadButtonText, fpStyleSelect, fpSizeSelect, fpCancelButton, fpCancelButtonClass, fpCancelButtonText, fpHeight, fpWidth) {
        var _this = this;
        this.listLabel = "Loading fonts...";
        this.directiveInstance = instance;
        this.directiveElementRef = elementRef;
        this.updateDialog(font, fpPosition, fpPositionOffset, fpPositionRelativeToArrow, fpPresetLabel, fpPresetFonts, fpUploadButton, fpUploadButtonClass, fpUploadButtonText, fpStyleSelect, fpSizeSelect, fpCancelButton, fpCancelButtonClass, fpCancelButtonText, fpHeight, fpWidth);
        this.service.getAllFonts('popularity').subscribe(function (data) {
            _this.loading = false;
            _this.googleFonts = data.items.map(function (font) { return _this.convertGoogleFont(font); });
            // Find styles for initial font
            var searchFont = _this.findFont(_this.initialFont.family, true);
            if (searchFont) {
                _this.font.files = searchFont.files;
                _this.font.styles = searchFont.styles;
                _this.loadGoogleFonts([_this.font]);
            }
            // Load Open Sans if available
            var openSans = _this.googleFonts.find(function (font) { return font.family == "Open sans"; });
            _this.loadGoogleFonts([openSans]);
        }, function (err) { return console.log(err); });
    };
    DialogComponent.prototype.updateDialog = function (font, fpPosition, fpPositionOffset, fpPositionRelativeToArrow, fpPresetLabel, fpPresetFonts, fpUploadButton, fpUploadButtonClass, fpUploadButtonText, fpStyleSelect, fpSizeSelect, fpCancelButton, fpCancelButtonClass, fpCancelButtonText, fpHeight, fpWidth) {
        this.font = font;
        this.styles = font.styles;
        this.initialFont = new interfaces_1.Font(font);
        this.fpPosition = fpPosition;
        this.fpPositionOffset = parseInt(fpPositionOffset);
        if (!fpPositionRelativeToArrow) {
            this.dialogArrowOffset = 0;
        }
        this.fpPresetLabel = fpPresetLabel;
        this.fpPresetFonts = fpPresetFonts;
        this.fpStyleSelect = fpStyleSelect;
        this.fpSizeSelect = fpSizeSelect;
        this.fpCancelButton = fpCancelButton;
        this.fpCancelButtonText = fpCancelButtonText;
        this.fpCancelButtonClass = fpCancelButtonClass;
        this.fpUploadButton = fpUploadButton;
        this.fpUploadButtonText = fpUploadButtonText;
        this.fpUploadButtonClass = fpUploadButtonClass;
        this.fpWidth = parseInt(fpWidth);
        this.fpHeight = parseInt(fpHeight);
        this.searchTerm.reset({ disabled: (this.fpPresetFonts.length > 0) });
        this.setDisplayedFontSource();
    };
    DialogComponent.prototype.openFontPicker = function () {
        if (!this.open) {
            this.setDialogPosition();
            this.searchTerm.setValue('');
            window.addEventListener('resize', this.listenerResize);
            document.addEventListener('mousedown', this.listenerMouseDown);
            this.open = true;
        }
    };
    DialogComponent.prototype.closeFontPicker = function () {
        this.open = false;
        window.removeEventListener('resize', this.listenerResize);
        document.removeEventListener('mousedown', this.listenerMouseDown);
    };
    DialogComponent.prototype.isFontAvailable = function (font) {
        if (!this.testWidth) {
            this.testContainer.style.fontFamily = 'monospace';
            document.body.appendChild(this.testContainer);
            this.testWidth = this.testContainer.clientWidth;
            document.body.removeChild(this.testContainer);
        }
        this.testContainer.style.fontFamily = font.family + ', monospace';
        document.body.appendChild(this.testContainer);
        var width = this.testContainer.clientWidth;
        document.body.removeChild(this.testContainer);
        return width != this.testWidth;
    };
    DialogComponent.prototype.getPresetFonts = function () {
        var _this = this;
        var presetFonts = [];
        if (this.googleFonts && this.fpPresetFonts && this.fpPresetFonts.length) {
            this.fpPresetFonts.forEach(function (font) {
                var fontClass = _this.findFont(font, true);
                if (!fontClass) {
                    fontClass = new interfaces_1.Font({
                        family: font,
                        size: 14,
                        style: "regular",
                        styles: ["regular", "italic", "700", "700italic"]
                    });
                }
                presetFonts.push(fontClass);
            });
            this.presetFonts = presetFonts;
            return presetFonts;
        }
    };
    DialogComponent.prototype.setDisplayedFontSource = function () {
        if (this.fpPresetFonts && this.fpPresetFonts.length) {
            this.setCurentFonts(this.getPresetFonts());
        }
        else {
            this.setCurentFonts(this.googleFonts);
        }
    };
    DialogComponent.prototype.setCurentFonts = function (target) {
        var _this = this;
        if (target != this.currentFonts) {
            this.currentFonts = target;
            this.loadedFonts = this.fontAmount;
            var initialFonts = this.currentFonts.slice(0, this.fontAmount);
            this.loadGoogleFonts(initialFonts);
            setTimeout(function () {
                _this.scrollbar.scrollTo(0);
            }, 0);
        }
    };
    DialogComponent.prototype.findFont = function (searchVal, exactMatch) {
        if (exactMatch === void 0) { exactMatch = false; }
        return this.findFonts(searchVal, exactMatch)[0];
    };
    DialogComponent.prototype.findFonts = function (searchVal, exactMatch) {
        if (exactMatch === void 0) { exactMatch = false; }
        searchVal = searchVal.toLowerCase();
        var fullmatchFonts = [];
        var candidateFonts = [];
        this.googleFonts.forEach(function (font) {
            if (searchVal === font.family.toLowerCase()) {
                fullmatchFonts.push(font);
                return;
            }
            if (exactMatch == false && font.family.toLowerCase().indexOf(searchVal) > -1) {
                candidateFonts.push(font);
            }
        });
        var resultFonts = fullmatchFonts.concat(candidateFonts);
        return resultFonts;
    };
    DialogComponent.prototype.loadMoreFonts = function () {
        var _this = this;
        if (this.open && !this.loading && this.loadedFonts < this.currentFonts.length) {
            var moreFonts = this.currentFonts.slice(this.loadedFonts, this.loadedFonts + this.fontAmount);
            this.loadGoogleFonts(moreFonts);
            this.loadedFonts += moreFonts.length;
            setTimeout(function () { _this.scrollbar.update(); }, 0);
        }
    };
    DialogComponent.prototype.loadGoogleFonts = function (fonts) {
        var _this = this;
        fonts.slice(0, this.fontAmount).forEach(function (font) {
            if (font && font.files && !_this.isFontAvailable(font)) {
                var style = font.styles.indexOf("regular") > -1 ? '' : ":" + font.styles.find(function (x) { return !isNaN(x); });
                try {
                    WebFont.load({
                        google: {
                            families: [font.family + ":" + style]
                        }
                    });
                }
                catch (e) {
                    console.warn("Problem with loading font:", font);
                }
            }
        });
    };
    DialogComponent.prototype.searchGoogleFonts = function (value) {
        if (!value) {
            this.onSearchReset();
            return;
        }
        value = value.toLowerCase();
        var searchResult = Array();
        if (this.googleFonts) {
            this.loadedFonts = this.fontAmount;
            searchResult = this.findFonts(value, false);
            this.setCurentFonts(searchResult);
        }
    };
    DialogComponent.prototype.convertGoogleFont = function (font) {
        var convertedFont = new interfaces_1.Font({
            family: font.family,
            styles: font.variants,
            files: font.files,
            style: null,
            size: 14
        });
        return convertedFont;
    };
    DialogComponent.prototype.onResize = function () {
        if (this.position === 'fixed') {
            this.setDialogPosition();
        }
        else {
            this.closeFontPicker();
        }
    };
    DialogComponent.prototype.onMouseDown = function (event) {
        if (!this.isDescendant(this.el.nativeElement, event.target) &&
            event.target != this.directiveElementRef.nativeElement) {
            this.closeFontPicker();
        }
    };
    DialogComponent.prototype.onUploadFiles = function () {
    };
    DialogComponent.prototype.onCancelSelect = function () {
        this.selectedFont = false;
        this.font.family = this.initialFont.family;
        this.font.size = this.initialFont.size;
        this.font.files = this.initialFont.files;
        this.font.style = this.initialFont.style;
        this.font.styles = this.initialFont.styles;
        this.closeFontPicker();
    };
    DialogComponent.prototype.onSelectFont = function (font) {
        this.selectedFont = true;
        this.font.family = font.family;
        this.font.styles = font.styles;
        this.font.files = font.files;
        this.font.style = font.styles.indexOf("regular") > -1 ? "regular" : font.styles[0];
    };
    DialogComponent.prototype.onSearchReset = function (event) {
        this.searchTerm.setValue('');
        this.setCurentFonts(this.googleFonts);
    };
    DialogComponent.prototype.onFontStyleChange = function ($event, font) {
        var str = this.font.family + ":" + $event.srcElement.value;
        if (font.files) {
            WebFont.load({
                google: {
                    families: [str]
                }
            });
        }
    };
    DialogComponent.prototype.isDescendant = function (parent, child) {
        var node = child.parentNode;
        while (node !== null) {
            if (node === parent) {
                return true;
            }
            node = node.parentNode;
        }
        return false;
    };
    DialogComponent.prototype.createDialogBox = function (element, offset) {
        return {
            top: element.getBoundingClientRect().top + (offset ? window.pageYOffset : 0),
            left: element.getBoundingClientRect().left + (offset ? window.pageXOffset : 0),
            width: element.offsetWidth,
            height: element.offsetHeight
        };
    };
    DialogComponent.prototype.setDialogPosition = function () {
        var parentNode = null;
        var position = 'static';
        var node = this.directiveElementRef.nativeElement;
        var dialogHeight = this.dialogElement.nativeElement.offsetHeight;
        while (node !== null && node.tagName !== 'HTML') {
            position = window.getComputedStyle(node).getPropertyValue("position");
            if (position !== 'static' && parentNode === null) {
                parentNode = node;
            }
            if (position === 'fixed') {
                break;
            }
            node = node.parentNode;
        }
        if (position !== 'fixed') {
            var boxDirective = this.createDialogBox(this.directiveElementRef.nativeElement, true);
            if (parentNode === null) {
                parentNode = node;
            }
            var boxParent = this.createDialogBox(parentNode, true);
            this.top = boxDirective.top - boxParent.top;
            this.left = boxDirective.left - boxParent.left;
        }
        else {
            var boxDirective = this.createDialogBox(this.directiveElementRef.nativeElement, false);
            this.top = boxDirective.top;
            this.left = boxDirective.left;
            this.position = 'fixed';
        }
        if (!this.fpWidth) {
            this.fpWidth = boxDirective.width;
        }
        if (!this.fpHeight) {
            this.fpHeight = boxDirective.height;
        }
        if (this.fpPosition === 'left') {
            this.top += boxDirective.height * this.fpPositionOffset / 100 - this.dialogArrowOffset;
            this.left -= this.fpWidth + this.dialogArrowSize;
        }
        else if (this.fpPosition === 'top') {
            this.top -= dialogHeight + this.dialogArrowSize;
            this.left += this.fpPositionOffset / 100 * boxDirective.width - this.dialogArrowOffset - 1;
            this.arrowTop = dialogHeight - 1;
        }
        else if (this.fpPosition === 'bottom') {
            this.top += boxDirective.height + this.dialogArrowSize;
            this.left += this.fpPositionOffset / 100 * boxDirective.width - this.dialogArrowOffset - 1;
        }
        else {
            this.top += boxDirective.height * this.fpPositionOffset / 100 - this.dialogArrowOffset;
            this.left += boxDirective.width + this.dialogArrowSize;
        }
    };
    DialogComponent.decorators = [
        { type: core_1.Component, args: [{
                    selector: 'font-picker',
                    template: '<div #dialogPopup [hidden]="!open" class="font-picker" [style.height.px]="fpHeight" [style.width.px]="fpWidth" [style.top.px]="top" [style.left.px]="left" [style.position]="position"><div class="arrow arrow-{{fpPosition}}" [style.top.px]="arrowTop"></div><div class="search-box"><input class="search-field" placeholder="Search from Google Web Fonts..." [formControl]="searchTerm" ><div class="search-reset" [hidden]="!searchTerm.value" (click)="onSearchReset($event)">×</div></div><div *ngIf="loading" class="load-text">Loading fonts list...</div><perfect-scrollbar #dialogScrollbar class="font-list" [config]="config"><div *ngIf="presetVisible && fpPresetFonts.length"><div class="font-group"><div class="group-text">{{fpPresetLabel}}</div><div class="group-line"></div></div><div *ngFor="let fontItem of presetFonts | StatefulSlice:0:loadedFonts" class="font-item" [ngClass]="{active: fontItem.family.toLowerCase() == font.family.toLowerCase(), selected: selectedFont}" [ngStyle]="{\'font-family\': fontItem.family}" (click)="onSelectFont(fontItem)"><div class="font-info"><span class="font-name">{{fontItem.family}}</span><div *ngIf="!fontItem.files" class="not-available">!</div></div></div></div><div class="font-group"><div class="group-text black">{{listLabel}}</div><div class="group-line black"></div></div><div *ngFor="let fontItem of (currentFonts | StatefulSlice:0:loadedFonts)" class="font-item" [ngClass]="{\'active\': fontItem.family.toLowerCase() == font.family.toLowerCase(), selected: selectedFont}" [ngStyle]="{\'font-family\': fontItem.family}" (click)="onSelectFont(fontItem)"><div class="font-info"><span class="font-name">{{fontItem.family}}</span></div></div></perfect-scrollbar><div *ngIf="selectedFont && (fpStyleSelect || fpSizeSelect)"><div class="font-options"><div *ngIf="fpStyleSelect" class="left"><select class="style-select" [disabled]="!font.styles" (change)="onFontStyleChange($event, font)" [(ngModel)]="font.style"><option *ngFor="let style of font.styles" [value]="style">{{style | FontStyles}}</option></select></div><div *ngIf="fpSizeSelect" class="right"><input type="number" pattern="[0-9]*" min="1" max="100" [ngClass]="{\'size-select\': fpStyleSelect}" [(ngModel)]="font.size" ></div></div></div><div *ngIf="fpUploadButton || fpCancelButton" class="button-area"><button *ngIf="fpUploadButton" type="button" class="{{fpUploadButtonClass}}" (click)="onUploadFiles()">{{fpUploadButtonText}}</button> <button *ngIf="fpCancelButton" type="button" class="{{fpCancelButtonClass}}" (click)="onCancelSelect()">{{fpCancelButtonText}}</button></div></div>',
                    styles: ['.font-picker{position:absolute;z-index:1000;top:250px;left:30px;display:flex;flex-direction:column;width:280px;height:320px;min-width:200px;min-height:240px;border:1px solid #777;cursor:default;user-select:none;font-family:\'Open Sans\',sans-serif;background-color:#fff}.font-picker[hidden]{display:none}.font-picker input,.font-picker select{width:100%;padding:4px;outline:0;border:none;border-bottom:2px solid #cfcfcf;font-size:14px;line-height:18px;text-align:left;background:0 0;transition:border .3s ease}.font-picker input:active,.font-picker input:focus,.font-picker input:hover,.font-picker select:active,.font-picker select:focus,.font-picker select:hover{border-bottom:2px solid #999}.font-picker input:invalid,.font-picker select:invalid{border-bottom:2px solid #e74c3c;box-shadow:none}.font-picker input::-webkit-inner-spin-button,.font-picker input::-webkit-outer-spin-button,.font-picker select::-webkit-inner-spin-button,.font-picker select::-webkit-outer-spin-button{margin:0;-webkit-appearance:none}.font-picker .arrow{position:absolute;z-index:999999;width:0;height:0;border-style:solid}.font-picker .arrow-left{top:10px;left:231px;border-width:5px 10px;border-color:transparent transparent transparent #777}.font-picker .arrow-right{top:10px;left:-20px;border-width:5px 10px;border-color:transparent #777 transparent transparent}.font-picker .arrow-top{top:0;left:10px;border-width:10px 5px;border-color:#777 transparent transparent}.font-picker .arrow-bottom{top:-20px;left:10px;border-width:10px 5px;border-color:transparent transparent #777}.font-picker .search-box{position:relative}.font-picker .search-field{box-sizing:border-box;padding:12px 48px 12px 16px}.font-picker .search-field:disabled{opacity:.5;cursor:not-allowed;background-color:#888}.font-picker .search-reset{position:absolute;top:0;right:0;padding:0 16px;cursor:pointer;color:#777;font-size:31px;transition:color .3s ease}.font-picker .search-reset:hover{color:#555}.font-picker .load-text{padding:16px;color:#000;font-size:12px;text-align:center}.font-picker .font-list{position:relative;overflow:hidden;flex:1 1 auto}.font-picker .font-list .font-item{position:relative;padding:8px 16px;cursor:pointer}.font-picker .font-list .font-item.active{font-weight:700}.font-picker .font-list .font-item.active.selected{cursor:default;font-weight:400;background:#e6e6e6}.font-picker .font-list .font-item .font-info{margin:4px 0}.font-picker .font-list .font-item .font-info .font-name{color:#555;font-size:14px;text-transform:capitalize}.font-picker .font-list .font-item .font-info .not-available{float:right;width:24px;margin:-8px -8px 0 0;color:#ef8b80;font-size:24px;font-weight:900;font-family:\'Open Sans\',sans-serif;text-align:center;text-transform:none}.font-picker .font-list .font-item .font-info .not-available::after{content:\'This font is not from Google Fonts. If you select it, you might have to inject it manually.\';position:absolute;z-index:10;top:0;right:12px;left:12px;opacity:0;padding:12px 16px;margin:0 auto;border-radius:2px;pointer-events:none;color:#fff;font-size:13px;font-weight:400;text-align:left;background:rgba(60,60,60,.8);transition:opacity 150ms ease}.font-picker .font-list .font-item .font-info .not-available:hover{color:#ea6153}.font-picker .font-list .font-item .font-info .not-available:hover::after{opacity:1}.font-picker .font-list .font-item .font-text{padding:4px 0;color:#999;font-size:16px;text-align:right}.font-picker .font-list .font-group{display:flex}.font-picker .font-list .font-group .group-line{flex:1 1 auto;height:2px;margin-top:14px;margin-right:12px;background:#e74c3c}.font-picker .font-list .font-group .group-line.black{background:#000}.font-picker .font-list .font-group .group-text{overflow:hidden;box-sizing:border-box;max-width:75%;padding:8px 12px;color:#e74c3c;font-size:11px;font-weight:700;text-align:left;white-space:nowrap;text-overflow:ellipsis}.font-picker .font-list .font-group .group-text.black{color:#000}.font-picker .font-options{display:flex;box-sizing:border-box;padding:8px;border-top:2px solid #cfcfcf}.font-picker .font-options .left{flex:1 1 auto;padding:0 2px}.font-picker .font-options .left .style-select{text-transform:capitalize}.font-picker .font-options .left .style-select:disabled{background-color:#777}.font-picker .font-options .right{padding:0 2px}.font-picker .font-options .right .size-select{width:48px;text-align:center}.font-picker .button-area{padding:0 4px 4px;border-top:1px solid #aaa;text-align:right}.font-picker .button-area .fp-cancel-button-class,.font-picker .button-area .fp-upload-button-class{display:inline-block;margin:8px;text-align:center;text-transform:uppercase}']
                },] },
    ];
    /** @nocollapse */
    DialogComponent.ctorParameters = function () { return [
        { type: core_1.ElementRef, },
        { type: font_picker_service_1.FontPickerService, },
    ]; };
    DialogComponent.propDecorators = {
        'dialogElement': [{ type: core_1.ViewChild, args: ['dialogPopup',] },],
        'scrollbar': [{ type: core_1.ViewChild, args: ['dialogScrollbar',] },],
    };
    return DialogComponent;
}());
exports.DialogComponent = DialogComponent;
//# sourceMappingURL=dialog.component.js.map