"use strict";
var core_1 = require('@angular/core');
var core_2 = require('@angular/core');
var interfaces_1 = require('./interfaces');
var dialog_component_1 = require('./dialog.component');
var FontPickerDirective = (function () {
    function FontPickerDirective(resolver, el, vc) {
        this.resolver = resolver;
        this.el = el;
        this.vc = vc;
        this.fpWidth = '280px';
        this.fpHeight = '320px';
        this.fpFallbackFont = new interfaces_1.Font({
            family: 'Roboto',
            size: 14,
            style: 'regular',
            styles: ['regular']
        });
        this.fpSizeSelect = true;
        this.fpStyleSelect = true;
        this.fpPosition = 'bottom';
        this.fpPositionOffset = '0%';
        this.fpPositionRelativeToArrow = false;
        this.fpCancelButton = false;
        this.fpCancelButtonText = 'Cancel';
        this.fpCancelButtonClass = 'fp-cancel-button-class';
        this.fpUploadButton = false;
        this.fpUploadButtonText = 'Upload';
        this.fpUploadButtonClass = 'fp-upload-button-class';
        this.fontPickerChange = new core_1.EventEmitter();
    }
    FontPickerDirective.prototype.ngOnInit = function () {
        var fontPicker = this.fontPicker;
        if (!this.fontPicker) {
            this.fontPicker = this.fpFallbackFont;
        }
        if (fontPicker != this.fontPicker) {
            this.fontPickerChange.emit(this.fontPicker);
        }
    };
    FontPickerDirective.prototype.onClick = function () {
        if (!this.dialog) {
            var compFactory = this.resolver.resolveComponentFactory(dialog_component_1.DialogComponent);
            var injector = core_2.ReflectiveInjector.fromResolvedProviders([], this.vc.parentInjector);
            this.dialog = this.vc.createComponent(compFactory, 0, injector, []).instance;
            this.dialog.setDialog(this, this.el, this.fontPicker, this.fpPosition, this.fpPositionOffset, this.fpPositionRelativeToArrow, this.fpPresetLabel, this.fpPresetFonts, this.fpUploadButton, this.fpUploadButtonClass, this.fpUploadButtonText, this.fpStyleSelect, this.fpSizeSelect, this.fpCancelButton, this.fpCancelButtonClass, this.fpCancelButtonText, this.fpHeight, this.fpWidth);
        }
        else if (!this.dialog.open) {
            this.dialog.updateDialog(this.fontPicker, this.fpPosition, this.fpPositionOffset, this.fpPositionRelativeToArrow, this.fpPresetLabel, this.fpPresetFonts, this.fpUploadButton, this.fpUploadButtonClass, this.fpUploadButtonText, this.fpStyleSelect, this.fpSizeSelect, this.fpCancelButton, this.fpCancelButtonClass, this.fpCancelButtonText, this.fpHeight, this.fpWidth);
            this.dialog.openFontPicker();
        }
        else {
            this.dialog.closeFontPicker();
        }
    };
    FontPickerDirective.decorators = [
        { type: core_1.Directive, args: [{
                    selector: '[fontPicker]',
                    host: {
                        '(click)': 'onClick()'
                    }
                },] },
    ];
    /** @nocollapse */
    FontPickerDirective.ctorParameters = function () { return [
        { type: core_2.ComponentFactoryResolver, },
        { type: core_2.ElementRef, },
        { type: core_2.ViewContainerRef, },
    ]; };
    FontPickerDirective.propDecorators = {
        'fontPicker': [{ type: core_1.Input, args: ['fontPicker',] },],
        'fpWidth': [{ type: core_1.Input, args: ['fpWidth',] },],
        'fpHeight': [{ type: core_1.Input, args: ['fpHeight',] },],
        'fpFallbackFont': [{ type: core_1.Input, args: ['fpFallbackFont',] },],
        'fpPresetLabel': [{ type: core_1.Input, args: ['fpPresetLabel',] },],
        'fpPresetFonts': [{ type: core_1.Input, args: ['fpPresetFonts',] },],
        'fpSizeSelect': [{ type: core_1.Input, args: ['fpSizeSelect',] },],
        'fpStyleSelect': [{ type: core_1.Input, args: ['fpStyleSelect',] },],
        'fpPosition': [{ type: core_1.Input, args: ['fpPosition',] },],
        'fpPositionOffset': [{ type: core_1.Input, args: ['fpPositionOffset',] },],
        'fpPositionRelativeToArrow': [{ type: core_1.Input, args: ['fpPositionRelativeToArrow',] },],
        'fpCancelButton': [{ type: core_1.Input, args: ['fpCancelButton',] },],
        'fpCancelButtonText': [{ type: core_1.Input, args: ['fpCancelButtonText',] },],
        'fpCancelButtonClass': [{ type: core_1.Input, args: ['fpCancelButtonClass',] },],
        'fpUploadButton': [{ type: core_1.Input, args: ['fpUploadButton',] },],
        'fpUploadButtonText': [{ type: core_1.Input, args: ['fpUploadButtonText',] },],
        'fpUploadButtonClass': [{ type: core_1.Input, args: ['fpUploadButtonClass',] },],
        'fontPickerChange': [{ type: core_1.Output, args: ['fontPickerChange',] },],
    };
    return FontPickerDirective;
}());
exports.FontPickerDirective = FontPickerDirective;
//# sourceMappingURL=font-picker.directive.js.map